/*
 * Copyright (c) 2010 Samsung Electronics Co., Ltd.
 *              http://www.samsung.com/
 *
 * S5PC110 - LCD Driver for U-Boot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <common.h>
#include <asm/io.h>
#include <asm/types.h>
#include <lcd.h>
#include <video_font.h>
#include <lcd.h>
#include "FONT.h"
#include <lcd_at070na.h>

DECLARE_GLOBAL_DATA_PTR;

#define LCD_BGCOLOR		0x1428A0
#define LCD_BPP                           LCD_COLOR8

static unsigned int gFgColor = 0xFF;
static unsigned int gLeftColor = LCD_BGCOLOR;

extern void LCD_Initialize_at070na(void);

void (*PutPixel)(u32 x, u32 y, u32 c);

#define Inp32(_addr)		readl(_addr)
#define Outp32(addr, data)	(*(volatile unsigned int *)(addr) = (data))
#define Delay(_a)		udelay(_a*1000)

u32 g_fbuffer_data_LCD[LCD_WIDTH * LCD_HEIGHT];

vidinfo_t panel_info = {
        .vl_col = 1024,
        .vl_row = 600,
        .vl_bpix = 3,
};

int lcd_line_length;
int lcd_color_fg;
int lcd_color_bg;

void *lcd_base;                         /* Start of framebuffer memory  */
void *lcd_console_address;              /* Start of console buffer      */

short console_col = 5;
short console_row = 5;



#if 1
void LCD_Initialize_at070na(void)
{
	u32 uFbAddr = CFG_LCD_FBUFFER;

	u32 i,uTmpReg;
	u32* pBuffer = (u32*)uFbAddr;

	printf("%s %d \n",__func__,__LINE__);

	LCD_setleftcolor(0x0);
	LCD_setfgcolor(0x0);
	LCD_setprogress(0);


	/*! by crazyboy 20140901 by treego
	 * Clock Setting - FIMD0(Display Controller 0)  
	 */


#if 1
	Outp32(0x1003C234,0x1116);
	Outp32(0x1003C534,0xfffff0);
	Outp32(0x1003C934,0xffffffc7);
#else
	uTmpReg = Inp32(0x1003C234); //CLK_SRC_LCD0

	uTmpReg = Inp32(0x1003C334); //CLK_SRC_MASK_LCD0

	uTmpReg = Inp32(0x1003C534); //CLK_DIV_LCD0

	uTmpReg = Inp32(0x1003C580); //CLKDIV2_RATIO

	uTmpReg = Inp32(0x1003C634); //CLK_DIV_STAT_LCD0

	uTmpReg = Inp32(0x1003C680); //CLKDIV2_STAT

	uTmpReg = Inp32(0x1003C934); //CLK_GATE_IP_LCD0

	uTmpReg = Inp32(0x1003C970); //CLK_GATE_BLOCK
//	printf("CLK_GATE_BLOCK = 0x%x\n",uTmpReg);



#endif

#if 0
	uTmpReg = Inp32(0x10010210);		// DISPLAY CONTROL  FIMD
	uTmpReg = (uTmpReg & ~(0x1 << 15)) | (1 << 15) ;
	Outp32(0x10010210, uTmpReg);

	uTmpReg = Inp32(0xf800027c);
	uTmpReg = (uTmpReg & ~(0x3)) | 0 ;
	Outp32(0xf800027c,uTmpReg);
#endif

	/*! by crazyboy 20140830 by treego
	 *  LCD Gpio Setting 
	 */
	Outp32(0x11400180, 0x22222222);	//set GPF0 as LVD_HSYNC,VSYNC,VCLK,VDEN,VD[3:0]
	Outp32(0x11400188,0x0);			//set pull-up,down disable
	Outp32(0x114001A0, 0x22222222);	//set GPF1 as VD[11:4]
	Outp32(0x114001A8,0x0);			//set pull-up,down disable
	Outp32(0x114001C0, 0x22222222);	//set GPF2 as VD[19:12]
	Outp32(0x114001C8,0x0);			//set pull-up,down disable
	Outp32(0x114001E0, 0x00222222);	//set GPF3 as VD[23:20]
	Outp32(0x114001E4,0x0);			//set pull-up,down disable

	Outp32(0x114000A0, 0x11111111);	//set GPD0 as Backlight
	Outp32(0x114000A8,0x0);			//set pull-up,down disable
	Outp32(0x114000A4,(1<<0)); //GPD0 DAT

	uTmpReg = Inp32(0x11000C00); 
	uTmpReg |= (0x1 << 28);
	Outp32(0x11000C00, uTmpReg);	//set GPX0[7]as LCD_PWREN
	Outp32(0x11000C08,0x0);			//set pulX2-up,down disable
	Outp32(0x11000C04,(1<<7));

	uTmpReg = Inp32(0x10010210); //DISPLAY_CONTROL
	uTmpReg |= (0x1 << 1) ;
	Outp32(0x10010210,uTmpReg);


	/*! by crazyboy 20140830 by treego
	 *  
	 */

	Outp32(VIDCON1, ((1 << BIT_HSYNC_INVERT) | (1 << BIT_VSYNC_INVERT))); //HSync,VSync Clock Polarity Inverted 
//	uTmpReg = Inp32(VIDCON1); //VIDCON1
//	printf("VIDCON1[0x%x] = 0x%x\n",VIDCON1,uTmpReg);

	Outp32(VIDTCON0, ((LCD_VBPD-1)<<BIT_VBPD)|((LCD_VFPD-1)<<BIT_VFPD)|(LCD_VSPW-1)<<BIT_VSPW);
	Outp32(VIDTCON1, ((LCD_HBPD-1)<<BIT_HBPD)|((LCD_HFPD-1)<<BIT_HFPD)|(LCD_HSPW-1)<<BIT_HSPW);
	Outp32(VIDTCON2, ((LCD_HEIGHT-1)<<BIT_LINEVAL)|(LCD_WIDTH-1)<<BIT_HOZVAL);

	Outp32(VIDINTCON0, (1 << BIT_SEL_WINDOWS0)); //Window 0 - Enable

	Outp32(BLENDCON, 0x0);

	//FrameBuffer Start/End Address Setting
	Outp32(VIDW00ADD0B0, uFbAddr + 0x0);
	Outp32(VIDW00ADD1B0, uFbAddr + (LCD_WIDTH*LCD_HEIGHT*4));
	Outp32(VIDW00ADD0B1, uFbAddr + 0x0);
	Outp32(VIDW00ADD1B1, uFbAddr + (LCD_WIDTH*LCD_HEIGHT*4));
	Outp32(VIDW00ADD0B2, uFbAddr + 0x0);
	Outp32(VIDW00ADD1B2, uFbAddr + (LCD_WIDTH*LCD_HEIGHT*4));
	Outp32(VIDW00ADD2, (LCD_WIDTH*4));

	//by crazyboy 20140830 by treego Check Register
	Outp32(WINCON0, 0x802c);
	Outp32(VIDOSD0B, ((LCD_WIDTH-1)<<11)|(LCD_HEIGHT-1));
	Outp32(VID0ALPHA0, 0xffffff);
	Outp32(VID0ALPHA1, 0xffffff);
	Outp32(WINCON0, 0x802d);
	Outp32(SHADOWCON, 0x1);

	//Outp32(VIDCON0, 0x193);
	Outp32(VIDCON0, 0x4c3);
	Outp32(VIDCON1, 0x82e0);
//	printf("VIDCON0[0x%x] = 0x%x\n",VIDCON0,Inp32(0x11C00000));

//	printf("%s %d \n",__func__,__LINE__);

}
#else
// "No LCD Type is defined!"
#endif

void _PutPixel32Bit(u32 x, u32 y, u32 c)
{
	u32* pBuffer = (u32*)CFG_LCD_FBUFFER;
    if(x < LCD_WIDTH && y < LCD_HEIGHT)
        //*(u32 *)(g_fbuffer_data_LCD + y * LCD_ORG_WIDTH + x) = c;
        *(u32 *)(pBuffer + y * LCD_WIDTH + x) = c;
}
/*
 */

#define N_BLK_VERT      2
#define N_BLK_HOR       3

static int test_colors[N_BLK_HOR*N_BLK_VERT] = {
        CONSOLE_COLOR_RED,      CONSOLE_COLOR_GREEN,    CONSOLE_COLOR_YELLOW,
        CONSOLE_COLOR_BLUE,     CONSOLE_COLOR_MAGENTA,  CONSOLE_COLOR_CYAN,
};

static void test_pattern (void)
{
        ushort v_max  = panel_info.vl_row;
        ushort h_max  = panel_info.vl_col;
        ushort v_step = (v_max + N_BLK_VERT - 1) / N_BLK_VERT;
        ushort h_step = (h_max + N_BLK_HOR  - 1) / N_BLK_HOR;
        ushort v, h;
        //uchar *pix = (uchar *)lcd_base;

        printf ("[LCD] Test Pattern: %d x %d [%d x %d]\n",
                h_max, v_max, h_step, v_step);

        /* WARNING: Code silently assumes 8bit/pixel */
        for (v=0; v<v_max; ++v) {
                uchar iy = v / v_step;
                for (h=0; h<h_max; ++h) {
                        uchar ix = N_BLK_HOR * iy + (h/h_step);
                        //*pix++ = test_colors[ix];
						PutPixel(v,h,test_colors[ix]);
                }
        }
}

void LCD_testpattern(void)
{
	test_pattern();
}
void LCD_turnon(void)
{
	lcd_line_length = (panel_info.vl_col * NBITS (panel_info.vl_bpix)) / 8;
	//printf("lcd_line_length = %d \n",lcd_line_length);
    PutPixel=_PutPixel32Bit;
 	lcd_base = (u32*)CFG_LCD_FBUFFER;
	gd->fb_base = lcd_base;
	LCD_Initialize_at070na();
}

void LCD_setfgcolor(unsigned int color)
{
	lcd_color_fg = color;
}

void LCD_setleftcolor(unsigned int color)
{
	lcd_color_bg = color;
}

void LCD_setprogress(int percentage)
{
	u32 i, j;
	u32* pBuffer = (u32*)CFG_LCD_FBUFFER;
#if 1
	for (i=0; i < (LCD_HEIGHT/100)*percentage; i++)
	{
		for (j=0; j < LCD_WIDTH; j++)
		{
			*pBuffer++ = lcd_color_fg;
		}
	}

	for (; i < LCD_HEIGHT; i++)
	{
		for (j=0; j < (LCD_WIDTH >> 5); j++)
		{
			*pBuffer++ = lcd_color_bg;
		}
		for (; j < LCD_WIDTH; j++)
		{
			*pBuffer++ = lcd_color_bg;
		}
	}
#endif

	//for(i=0;i<LCD_HEIGHT;i++)
	//	for(j=0;j<LCD_WIDTH;j++)
	//		*pBuffer++ = gFgColor;
}


void lcd_setcolreg (ushort regno, ushort red, ushort green, ushort blue)
{
	printf("%s..\n",__func__);
 
}
void lcd_enable(void)
{
	printf("%s..\n",__func__);

}

void lcd_disable(void)
{
	printf("%s..\n",__func__);

}

void lputs(u32 x, u32 y, u32 c, char *str)
{
	int i,j,k;
	int oneLine;

	for(i=0;i<strlen(str);i++)
	{
		for(j=0; j<16; j++)
		{
			if ((str[i]>=0x20)&&(str[i]<0x80))
				oneLine=chr_font8X16[str[i]-0x20][j];
			else
				oneLine=0;

			for(k=0;k<8;k++)
			{
				if(oneLine&(1<<k))
					PutPixel(x+7-k,y+j,c);
			}
		}
		x+=8;
	}
}
void lprintf(u32 x, u32 y, u32 c, const char *fmt,...)
{
	va_list ap;
	char string[256];

	va_start(ap,fmt);
	vsprintf(string,fmt,ap);
	lputs(x,y,c,string);
	va_end(ap);
}

void LCD_set_fb_tmp_test(ulong command, ulong value1, ulong value2, ulong value3)
{
    int i;

	printf ("LCD_set_fb_tmp_test Command:%d, value1:%d, value2:%d, value3:%d\n",
            command, value1, value2, value3);

    switch(command)
    {
    case 0:
        printf ("Command 0 - Test Value\n");
		LCD_turnon();
		LCD_setfgcolor(value1);
		LCD_setleftcolor(value2); //CRZ icanjji
		for(i = 0 ; i < 100; i++)
			LCD_setprogress(i);
        break;

    case 1:
    	printf ("Command q - char print\n");
        lprintf(0, 0, value1, "1234567890AB");
        break;

    case 2:
    	printf ("Command 2 - test_pattern \n");
		test_pattern();
        break;

    case 3:
    	printf ("Command 3 - test_pattern \n");
		LCD_turnon();
		LCD_setfgcolor(value1);
		LCD_setleftcolor(value2); //CRZ icanjji
		for(i = 0 ; i < 100; i++)
			LCD_setprogress(i);
        lprintf(420, 400, value3, "1234567890AB");
        break;
    default:
    	printf ("lcd_control Command fail\n");
        break;
    }

}

int
do_lcd_control (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong command, value1, value2, value3;
	char **av;

//    lcd_width_height_setting();
//	printf("%s %d \n",__func__,__LINE__);

	av = argv + 1;

	command = simple_strtoul(*av++, NULL, 16);
	value1 = simple_strtoul(*av++, NULL, 16);
	value2 = simple_strtoul(*av++, NULL, 16);
	value3 = simple_strtoul(*av++, NULL, 16);

    //LCD_set_fb_initialData();
    LCD_turnon();
    LCD_set_fb_tmp_test(command, value1, value2, value3);
	return 0;
}

U_BOOT_CMD(
	lcd,	5,		1,	do_lcd_control,
	"lcd [cmd] [color1-fg] [color2-bg] [color3-font] - lcd control\n",
	"lcd [cmd] [color1-fg] [color2-bg] [color3-font] - lcd control\n"

);
